/* Copyright (C) Bill Buckels 2008 */

/* whenever mucking with the keyboard directly in an Aztec C64
   program, getchar() needs to be called at least once in
   that program in order for Aztec C to exit cleanly
   evidently because of the way console IO has been set-up
   to work in the Aztec C environment.

   getchar() has a couple of potentially nasty effects
   if used throughout an Aztec C program in that it translates
   the C64 navigation keys and can really muck-up your screens,
   and in the prompt which does not survive either the mixed-case
   conversion (it prints an uppercase 'Q' instead of a ball),
   and in graphics modes (where it is best to clear the screen and
   home the cursor) it prints its prompt character into vram
   (of course) and mucks-up the top left corner with a bogus
   color.

   Since you are not going to want the user mucking around in
   your program screens anyway with no benefit to your design or their
   use of your program I'd suggest just using both getchar() and gets()
   as a quick and dirty input mechanism.

   If you do not use getchar() or gets() then stuff a non-volatile
   value (like a carriage return) into the keyboard right at the end of
   your program, before clearing your screen (putchar(147)) then call
   exit(0).

   */


#include <poke.h>

/* low level keyboard routine */
/* returns first character in waiting in keyboard buffer */
/* if keyboard is busy */
/* returns 0 if not */
int kbhit()
{
	int ch;

	ch = kpress[0]; /* get keypress */

	if (ch!= 0x40) ch = kbuf[0];
	else ch = 0;

	return ch;

}





