
/* btest.c (C) Copyright Bill Buckels 2008 */

/* an Aztec C64 demo using a converted text screen from
   the IBM created by bsv2x64.

   KISS 101 (Keep it Simple Simon)

   all the programmer needs to do is draw the
   screen on the IBM remembering to keep it 40 columns
   wide x 25 rows.

   It's just like baking a cake. Recipe below.

   See bsv2x64.c for details or use bsv2x64.exe
   without details. See MAKEFILE and makedisk.bat
   for usual build directions.

   Runs on a C64 or in an C64 emulator.

   */



/* the CMAIN define must always be the first line
   in your main program when
   using poke.h and B64NAT.LIB and nowhere else.

   B64NAT.LIB makes extensive use of the memory variables
   in poke.h which are included in your main program
   and nowhere else.

   They are referenced as externals by B64NAT.LIB and any
   overlay modules that you may create.

   when linking with B64NAT.LIB the aztec C linker requires
   B64NAT.LIB on the link line before C64NAT.LIB to
   resolve B64NAT.LIB calls to C64NAT.LIB
*/

#define CMAIN 1

#include <poke.h>
#include <colors.h>


/* character array created from IBM text screen btest.bsv */
/* array structure is C64 screen value, C64  color value */
/* char btest[2000]; */

#include "btest.h"

char *prompt = "Press 0-9 and A-F for Color.";

main()
{

   int row, col1, col2, color, reverse=1, normal=0;
   int ch, color, len;
   int prow = 12, pcol = 5;
   char buf[10];


   /* first a simple example */
   /* this one gets the C64 Aztec C good houskeeping seal of approval */

   scr_mixed(); /* put the C64 in mixed case mode */
   scr_clear(); /* clear screen */

   memscr(btest); /* load the text screen array */

   outtext("Press A Key To See Colors...",prow,pcol,C64_BWHITE,normal);

   /* clear all keypressess */
   kbflush();

   /* wait until a key is pressed */
   while(!kbhit());

   scr_back(0, -1); /* set background to black */
   scr_clear(); /* clear screen */


   row = 0;
   col1 = 0;   /* normal column */
   col2 = 10;  /* reverse column */
   for (color = 1; color < 16; color++) {
     sprintf(buf,"Color %2d", color);
     outtext(buf,row,col1,color,normal);
     outtext(buf,row,col2,color,reverse);
     row++;
   }
   len = strlen(buf); /* save the string length */

   prow = 16;
   pcol = 0;
   outtext("Press Any Key to Reverse...",prow,pcol,C64_BWHITE,normal);

    /* wait until a key is pressed */

    while (kbhit()); /* wait for key release */
    kbflush();
    getch();  /* wait for another key press */

   row = 0;
   for (color = 1; color < 16; color++) {
     revtext(row,col1,len,reverse);
     revtext(row,col2,len,normal);
     row++;
   }


	outtext(prompt,prow,pcol,C64_BWHITE,normal);
	outtext("Press 'X' To Load Screen...",prow+1,pcol,C64_BWHITE,normal);
    len = strlen(prompt);

TOP:;


    ch = toupper(getch());

    /* characters 0-9 and A-F */
    if ((ch > 47 && ch < 58) || (ch > 64 && ch < 71)) {
		if (ch < 65) color = ch - 48;
		else color = ch - 55;
		attext(prow,pcol,len,color);
		/* border color,background color */
        scr_back(-1,color); /* set border only */
	}

	if (ch != 'X') goto TOP;

    scr_back(-1,C64_LBLUE); /* reset border only */
    scr_clear(); /* clear screen */

   /* load a binary screen - use default load addresses */
   /* second arg is set to 0 - otherwise override addr */
   bload("BTEST.BXC",0);
   bload("BTEST.BXD",0);


   prow = 12;
   pcol = 9;
   outtext("Press A Key To Exit...",prow,pcol,C64_BWHITE,normal);
   kbflush();
   while(!kbhit());

   /* eat the last key pressed */
   getchar();
   scr_clear(); /* clear screen to hide getchar */
   scr_reset(); /* clear again and reset to default and UPPER case */

   exit(0);

}

