
/* dirlist.c (C) Copyright 2008 Bill Buckels */
/* Read a file list from the current disk directory
   in an Aztec C Commodore 64 program */

#define CMAIN 1

#include <poke.h>
#include <colors.h>


/* The linker provides a 2K memory hole directly below
   the graphics screen area which provides a static
   buffer without the need for memory allocation.

   This is where we load the disk directory (at decimal 6144).

   This is also where I preload my fonts and title screen
   vram in an image loader program.

   The linker line from the MAKEFILE is as follows:

   ln65 $(PRG).rel C64NAT.LIB -b 810 -d 4400 -c 5000

   You should also note that the base address of 810
   provides the usual jmp address for the launcher code
   that is appended to the beginning of the PRG
   after linking. (SYS 2064 CALL)

   The data is well beyond the screen
   and allows 3K before loading the program code.
   The deal here is to make everything fit
   without clobbering BASIC and the rest of the C64.

   When that occurs we just go to overlays and
   you can review my overlay samples to see how we
   do that. 'Nuff said.

*/

#define FOFF 17

main()
{
	int idx, jdx, found;

    scr_clear(); /* home cursor and clear screen */
    scr_txtcolor(C64_BWHITE); /* change text to bwhite */

    printf("LISTING OF CURRENT DISK:\n");

    /* read the directory of the disk that we are on */
	found = dlist("DIRLIST.PRG","*");

	printf("%d FILE(S) FOUND:\n",found);

	for (idx = 0; idx < found; idx++) {
	   jdx = (idx * FOFF); /* each listing is 17 bytes long */
	   puts(&dirptr[jdx]);
    }

    found = dlist("DIRLIST.PRG","PRG");
    printf("%d PRG(S) FOUND:\n",found);
    for (idx = 0; idx < found; idx++) {
	   jdx = (idx * FOFF); /* each listing is 17 bytes long */
	   puts(&dirptr[jdx]);
    }

    printf("PRESS A KEY TO END ");
    getchar();
    scr_txtcolor(C64_LBLUE); /*  change text to lblue */
    scr_clear(); /* clear screen and outa here */
	exit(0);
}
